﻿using System;
using System.Web;
using System.Web.UI;
using System.Collections.Generic;
using System.Text;
using System.IO;
using System.Globalization;

public partial class AdminPanel : Page
{
	protected int _pluginID;
	protected List<PluginInfo> _installedPlugins = new List<PluginInfo>();
    public bool allowResize;
    public string mode;

    public string Login;
    public string Password;
    public string Host;
    public string Port;

	protected string _contentClass;

    public AdminPanel()
    {
        _pluginID = 0;
    }

    protected void Page_Load(object sender, EventArgs e)
	{
        Session.Timeout = 120;

		_contentClass = "wm_content_server";
		if (Utils.IsPluginExist("WebMailLite"))
		{
			_contentClass = Utils.IsPluginExist("Server") ? "wm_content_serverlite" : "wm_content_webmaillite";
		}
		if (Utils.IsPluginExist("WebMail"))
		{
			_contentClass = Utils.IsPluginExist("Server") ? "wm_content_server" : "wm_content_webmail";
		}

        try
        {
            if (Session["AUTH"] == null)
            {
                Response.Redirect("Login.aspx");
                return;
            }
            
            AdminPanelSettings apSettings = new AdminPanelSettings();
            apSettings.LoadAdminPanelSettings();
            Login = apSettings.User;
            Password = apSettings.Pass;
            Host = apSettings.Host;
            Port = apSettings.Port;

            if (!string.IsNullOrEmpty(Request.QueryString["mode"]))
                mode = Request.QueryString["mode"];

            string rootFolder = Page.MapPath("");

            if (string.Compare(mode, "show_log", true, CultureInfo.InvariantCulture) == 0)
            {
                Response.Clear();
                Response.AddHeader("Content-Type", "text/html");
                Response.Write(@"<html><body><pre class=""wm_print_content"">");
                string logPath = WebMailPro.Utils.GetLogFilePath();
                Response.Write(HttpUtility.HtmlEncode(WebMailPro.Utils.LoadFromFile(logPath, Encoding.Default)));
                Response.Write("</pre></body></html>");
                Response.Flush();
                Response.Close();
            }
            else if (string.Compare(mode, "show_log_partial", true, CultureInfo.InvariantCulture) == 0)
            {
                Response.Clear();
                Response.AddHeader("Content-Type", "text/html");
                Response.Write(@"<html><body><pre class=""wm_print_content"">");
                string logPath = WebMailPro.Utils.GetLogFilePath();
                FileInfo fi = new FileInfo(logPath);
                string log = string.Empty;
                const int lastPartSize = 20000;
                if (fi.Exists)
                {
                    log = fi.Length > lastPartSize ? (string)WebMailPro.Utils.LoadFromFile(logPath, Encoding.Default, (int)fi.Length - lastPartSize, lastPartSize) : WebMailPro.Utils.LoadFromFile(logPath, Encoding.Default, 0, lastPartSize);
                }
                Response.Write(log);
                Response.Write("</pre></body></html>");
                Response.Flush();
                Response.Close();
            }

            _installedPlugins = Utils.GetPluginInfos(rootFolder);
            LoadState();
            if (!Page.IsPostBack)
            {
                if (!string.IsNullOrEmpty(Request.QueryString["plugin"]))
                {
                    int.TryParse(Request.QueryString["plugin"], out _pluginID);
                    if ((_pluginID < 0) || (_pluginID > _installedPlugins.Count))
                        _pluginID = 0;
                }
                if (!string.IsNullOrEmpty(Request.QueryString["mode"]) && Request.QueryString["mode"] == "logout")
                {
                    Response.Redirect("Logout.aspx");
                }

                if (!string.IsNullOrEmpty(Request.QueryString["mode"]) && Request.QueryString["mode"] == "help")
                {
                    Session.Abandon();
                    Response.Redirect("docs/index.htm");
                }
            }
            LoadPluginContent();
            SaveState();
        }
        catch (Exception ex)
        {
            Utils.SetPageErrorMessage(Page, ex.Message);
        }
	}

	protected string OutputHeader()
	{
		StringBuilder result = new StringBuilder();
		for (int i = 0; i < _installedPlugins.Count; i++)
		{
			result.AppendFormat(@"
					<div id=""PluginHeaderID{0}"" class=""{1}"">
						<a href=""adminpanel.aspx?plugin={2}"">{3}</a>
					</div>
", i, (_pluginID == i) ? "wm_accountslist_email_activ" : "wm_accountslist_email", i, _installedPlugins[i].Caption);
		}
        result.Append(@"
        <div class=""wm_accountslist_logout"">
            <a href=""Logout.aspx"">Logout</a>
        </div>
        ");
        result.Append(@"
        <div class=""wm_accountslist_logout"">
            <a href=""adminpanel.aspx?mode=help"">Help</a>
        </div>
        ");
        return result.ToString();
	}

	protected void LoadPluginContent()
	{
	    if (_pluginID < _installedPlugins.Count)
	    {
	        if ((_installedPlugins[_pluginID].FolderName == "Domains") ||
	            (_installedPlugins[_pluginID].FolderName == "Users"))
	            allowResize = true;
	        else
	            allowResize = false;

            if (Utils.IsPluginExist(_installedPlugins[_pluginID].FolderName))
            {
                Control ctrl =
                    LoadControl(string.Format("Plugins/{0}/main.ascx", _installedPlugins[_pluginID].FolderName));

                MainPlugin plugin = ctrl as MainPlugin;

                if (ctrl != null && plugin.PluginID == _installedPlugins[_pluginID].FolderName)
                {
                    ctrl.ID = string.Format("PluginContentID{0}", _pluginID);
                    contentPlaceHolder.Controls.Add(ctrl);
                }
            }
	    }
	}

	protected void SaveState()
	{
		Session["pluginID"] = _pluginID;
	}

	protected void LoadState()
	{
		if (Session["pluginID"] != null) int.TryParse(Session["pluginID"].ToString(), out _pluginID);
	}
}
