using System;
using System.IO;
using System.Web;

namespace WebMailLite
{
	public enum WMStartPage
	{
		Mailbox = 0,
		NewMessage = 1,
	}

	/// <summary>
	/// Summary description for Integration.
	/// </summary>
	public class Integration
	{
		protected WebmailResourceManager _resMan = null;
		protected string _wmRoot = string.Empty;

		public Integration(string wmRootPath)
		{
			HttpApplicationState app = HttpContext.Current.Application;
			if (app != null)
			{
				if (app[Constants.appSettingsDataFolderPath] == null)
				{
					app.Add(Constants.appSettingsDataFolderPath, Utils.GetDataFolderPath());
				}
			}
			_resMan = (new WebmailResourceManagerCreator()).CreateResourceManager();
			_wmRoot = wmRootPath;
		}

		public void UserLoginByEmail(string email, string login, string password, WMStartPage startPage, string toEmail)
		{
			Account acct = Account.LoginAccount(email, login, password);
			if (acct != null)
			{
				if (HttpContext.Current != null)
				{
					FileSystem fs = new FileSystem(acct.Email, 0, false);
					string hashString = Utils.GetMD5DigestHexString(HttpContext.Current.Session.SessionID);
					if (fs.SaveAccount(acct, hashString))
					{
						HttpContext.Current.Response.Redirect(Path.Combine(_wmRoot, string.Format(@"integration/integr.aspx?email={2}&h={3}&scr={0}&to={1}", (int)startPage, toEmail, acct.Email, hashString)));
					}

				}
			}
		}

		public void UserLoginByEmail(string email, string login, string pass, WMStartPage startPage)
		{
			UserLoginByEmail(email, login, pass, startPage, null);
		}

		public void UserLoginByEmail(string email, string login, string pass)
		{
			UserLoginByEmail(email, login, pass, WMStartPage.Mailbox, null);
		}

		public void UserLogin(string email, string login, string pass, string incHost,
			string outHost, int incPort, int outPort, bool useAuth, WMStartPage startPage, string toEmail)
		{	
			Account acct = Account.LoginAccount(email, login, pass, incHost, incPort, outHost, outPort, useAuth, true);
			if (acct != null)
			{
				if (HttpContext.Current != null)
				{
					FileSystem fs = new FileSystem(acct.Email, 0, false);
					string hashString = Utils.GetMD5DigestHexString(HttpContext.Current.Session.SessionID);
					if (fs.SaveAccount(acct, hashString))
					{
						HttpContext.Current.Response.Redirect(Path.Combine(_wmRoot, string.Format(@"integration/integr.aspx?email={2}&h={3}&scr={0}&to={1}", (int)startPage, toEmail, acct.Email, hashString)));
					}

				}
			}
		}

		public void UserLogin(string email, string login, string pass, string incHost,
			string outHost, int incPort, int outPort, bool useAuth, WMStartPage startPage)
		{	
			UserLogin(email, login, pass, incHost, outHost, incPort, outPort, useAuth, startPage, null);
		}

		public void UserLogin(string email, string login, string pass, string incHost,
			string outHost, int incPort, int outPort, bool useAuth)
		{	
			UserLogin(email, login, pass, incHost, outHost, incPort, outPort, useAuth, WMStartPage.Mailbox, null);
		}
	}
}